/* can.c */

#include "can.h"

CAN _can;
CAN* can = &_can;


/* -------------------------------------------------------------------------- */
/* CAN EventHandler
*/
void CanInit(void){
  can->Intr0 = 0;
  can->CanSetIntr0 = CanSetIntr0;
  can->CanResetIntr0 = CanResetIntr0;
  
  CanConfig();
}
void CanSetIntr0() { can->Intr0 = 1; }
void CanResetIntr0() { can->Intr0 = 0; }



/* -------------------------------------------------------------------------- */
/* CAN function
   - ISO 11898:2003 standard protocol
   - Paramter bit fields
      1) 11 bit type
         SOF | 11bit ID | RTR | ODE | r0 | DLC | 0~8 bytes Data | CRC | ACK | EOF | IFS
      2) 29 bit type
         SOF | 11bit ID | SRR | ODE | 18 bit ID | RTR | r0 | r1 | DLC | 0~8 bytes Data | CRC | ACK | EOF | IFS
      #Package unit description
        SOF: Start of frame
        Identifier: 11-bit identifier to set the priority of the message. The lower the value, the higher the priority.
        RTR: single remote transmission request(Define when you need information from other nodes, Data Frame: 0, Remote Frame:1)
        IDE: dominant single identifier extension(Standard: 0, Extended: 1)
        r0: Reserved bit
        DLC: data length code
        Data: 0~8 bytes
        CRC: Cyclic redundancy check (16bit checksum)
        EOF: End of Frame
        IFS: interframe space (7bit)
        SRR: substitute remote request
        IDE: identifier extension
        r1: Reserved bit
*/

void CanConfig(void)
{
  /* 1. Set the clock of CAN function. */
  RCC_APB1PeriphClockCmd((RCC_APB1Periph_CAN1 | RCC_APB1Periph_CAN2), ENABLE);
  
  /* 2. Set the interrupt of CAN function. */
  /*
  NVIC_InitTypeDef NVIC_InitStructure;
  //NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
  */
  
  /* 3. Initialize the CAN function.*/
  CAN_InitTypeDef        CAN_InitStructure;
  CAN_FilterInitTypeDef  CAN_FilterInitStructure;

  /* CAN register init */
  CAN_DeInit(CAN2);
  
  /* CAN cell init */
  CAN_InitStructure.CAN_TTCM = DISABLE;
  CAN_InitStructure.CAN_ABOM = DISABLE;
  CAN_InitStructure.CAN_AWUM = DISABLE;
  CAN_InitStructure.CAN_NART = DISABLE;
  CAN_InitStructure.CAN_RFLM = DISABLE;
  CAN_InitStructure.CAN_TXFP = DISABLE; 
#if CAB_LOOPBACK_TEST
  CAN_InitStructure.CAN_Mode=CAN_Mode_LoopBack;
#else
  CAN_InitStructure.CAN_Mode=CAN_Mode_Normal;
#endif
  CAN_InitStructure.CAN_SJW = CAN_SJW_1tq;
  
  /* 3-4. Set the baudrate */
  /* CAN Baudrate = 175kbps (CAN clocked at 36 MHz) */
  CAN_InitStructure.CAN_BS1 = CAN_BS1_4tq;
  CAN_InitStructure.CAN_BS2 = CAN_BS2_2tq;
  CAN_InitStructure.CAN_Prescaler = 9;
  CAN_Init(CAN2, &CAN_InitStructure);
  
  /* 3-4. Set the filter */
  CAN_FilterInitStructure.CAN_FilterNumber = 14;
  CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;
  CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;
  CAN_FilterInitStructure.CAN_FilterIdHigh = 0x0000;
  CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;
  CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0000;
  CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;  
  CAN_FilterInitStructure.CAN_FilterFIFOAssignment = 0;
  CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
  CAN_FilterInit(&CAN_FilterInitStructure);

  #if CAB_LOOPBACK_TEST
    CanLoopbackTest();
  #else
    /* NOP */
  #endif
}

void CanSend(unsigned char id,  unsigned char* buf,  unsigned char len){
  
  CanTxMsg TxMessage;
  unsigned int i = 0;
  uint8_t TransmitMailbox = 0;
  
  TxMessage.StdId=id;
  TxMessage.RTR=CAN_RTR_DATA;
  TxMessage.IDE=CAN_ID_STD;
  TxMessage.DLC=2;
  for(i=0;i<len;i++) TxMessage.Data[i]=buf[i];
  
  TransmitMailbox=CAN_Transmit(CAN2, &TxMessage);
  
  i = 0; /* Timeout */
  while((CAN_TransmitStatus(CAN2, TransmitMailbox) != CANTXOK) && (i != 0xFFFF))
  {
    i++;
  }
  
  
  while((CAN_MessagePending(CAN2, CAN_FIFO0) < 1) && (i != 0xFFFF))
  {
    i++;
  }
  
}

void CanLoopbackTest(void)
{
  CanTxMsg TxMessage;
  CanRxMsg RxMessage;
  uint32_t i = 0;
  uint8_t TransmitMailbox = 0;

  /* transmit */
  TxMessage.StdId=0x11;
  TxMessage.RTR=CAN_RTR_DATA;
  TxMessage.IDE=CAN_ID_STD;
  TxMessage.DLC=2;
  TxMessage.Data[0]=0xCA;
  TxMessage.Data[1]=0xFE;

  TransmitMailbox=CAN_Transmit(CAN2, &TxMessage);
  i = 0;
  while((CAN_TransmitStatus(CAN2, TransmitMailbox) != CANTXOK) && (i != 0xFFFF))
  {
    i++;
  }

  i = 0;
  while((CAN_MessagePending(CAN2, CAN_FIFO0) < 1) && (i != 0xFFFF))
  {
    i++;
  }

  /* receive */
  RxMessage.StdId=0x00;
  RxMessage.IDE=CAN_ID_STD;
  RxMessage.DLC=0;
  RxMessage.Data[0]=0x00;
  RxMessage.Data[1]=0x00;
  CAN_Receive(CAN2, CAN_FIFO0, &RxMessage);

  if (RxMessage.StdId!=0x11)
  {
    while(1) {;}
  }

  if (RxMessage.IDE!=CAN_ID_STD)
  {
    while(1) {;}
  }

  if (RxMessage.DLC!=2)
  {
    while(1) {;}
  }

  if ((RxMessage.Data[0]<<8|RxMessage.Data[1])!=0xCAFE)
  {
    while(1) {;}
  }
}


void USB_LP_CAN1_RX0_IRQHandler(void)
{
  CanSetIntr0();
  
  /* example ===================================================================

  if(can->CanIntr0)
  {
    ResetCanIntr0();

    CanRxMsg RxMessage;
    RxMessage.StdId=0x00;
    RxMessage.ExtId=0x00;
    RxMessage.IDE=0;
    RxMessage.DLC=0;
    RxMessage.FMI=0;
    RxMessage.Data[0]=0x00;
    RxMessage.Data[1]=0x00;

    CAN_Receive(CAN1, CAN_FIFO0, &RxMessage);

    if((RxMessage.ExtId==0x1234) && (RxMessage.IDE==CAN_ID_EXT)
       && (RxMessage.DLC==2) && ((RxMessage.Data[1]|RxMessage.Data[0]<<8)==0xDECA))
    {
      while(1) {;}
    }
  }
  */
}

