/* f103bsp.c */

#include "f103bsp.h"


/* -------------------------------------------------------------------------- */
/* Board Support Package 
*/
BSP _bsp;
BSP* bsp = &_bsp;

void BspInit(void){
  bsp->Timer0 = 0;
  bsp->Timer1 = 0;
  bsp->SetTimer0 = SetTimer0;
  bsp->SetTimer1 = SetTimer1;
  bsp->ResetTimer0 = ResetTimer0;
  bsp->ResetTimer1 = ResetTimer1;
}
void SetTimer0() { bsp->Timer0 = 1; }
void SetTimer1() { bsp->Timer1 = 1; }
void ResetTimer0() { bsp->Timer0 = 0; }
void ResetTimer1() { bsp->Timer1 = 0; }


/* -------------------------------------------------------------------------- */
/* Timer 1~3 
    - Timer 2, Use 'Bsp::Timer0'
*/
void TIM2_IRQHandler()
{
    TIM2->SR=~0x0001;       //   TIM2 Interrupt flag clear
    
    SetTimer0();
}

#if 0 /* 8MHz Version */
void TimerInit(void)
{
    /* STM32F103RE*/
    RCC->APB2ENR=0x00000008; //  PB Clock EN
    GPIOB->CRL=0x00000001;   //  PB0 Output
    RCC->APB1ENR=0x00000001; //  TIM2 Clock EN
    /*
      100ms     TIM2->PSC=900-1       TIM2->ARR=900-1
      50ms      TIM2->PSC=900-1       TIM2->ARR=450-1
      10ms      TIM2->PSC=900-1       TIM2->ARR=90-1
      5ms       TIM2->PSC=449-1       TIM2->ARR=90-1
    */
    TIM2->PSC=800-1;        //  Prescaler
    TIM2->ARR=500-1;        //  Period 1KHz
    TIM2->EGR=0x0001;        //  Update generation
    TIM2->DIER=0x0001;       //  Update interrupt EN
    TIM2->CR1=0x0001;        //  Up Counter Counter EN
    
    SCB->AIRCR=0x00000300;   //  Group 74 Sub None
    NVIC->IP[28]=0xF0;       //  TIM2 IRQ Number [28] 0xF0 = Priority group 15
    NVIC->ISER[0]=0x10000000;//  TIM2 Interrupt SET [28]0x05==[0] 1(IRQn&0x1F)
}
#endif

void TimerInit(void)
{
    /* STM32F103RE*/
    RCC->APB2ENR=0x00000008; //  PB Clock EN
    GPIOB->CRL=0x00000001;   //  PB0 Output
    RCC->APB1ENR=0x00000001; //  TIM2 Clock EN
    /*
      PCK1 Timer Clock == 60MHz
      100ms     TIM2->PSC=6000-1       TIM2->ARR=1000-1
      50ms      TIM2->PSC=3000-1       TIM2->ARR=1000-1
      10ms      TIM2->PSC=6000-1       TIM2->ARR=100-1
      5ms       TIM2->PSC=3000-1       TIM2->ARR=100-1
    */
    TIM2->PSC=3000-1;        //  Prescaler
    TIM2->ARR=1000-1;        //  Period 1KHz
    TIM2->EGR=0x0001;        //  Update generation
    TIM2->DIER=0x0001;       //  Update interrupt EN
    TIM2->CR1=0x0001;        //  Up Counter Counter EN
    
    SCB->AIRCR=0x00000300;   //  Group 74 Sub None
    NVIC->IP[28]=0xF0;       //  TIM2 IRQ Number [28] 0xF0 = Priority group 15
    NVIC->ISER[0]=0x10000000;//  TIM2 Interrupt SET [28]0x05==[0] 1(IRQn&0x1F)
}


/* -------------------------------------------------------------------------- */
/* UserClockSetting
    - Custom Clock, for CAN 
      1) SysClock  => 60MHz
      2) HCLK      => 60MHz
      3) APB1Periph PCLK1 Peripheral Clock => 30 MHz
      4) APB1Periph PCLK1 Timer Clock      => 60 MHz
*/
void UserClockSetting()
{
  /* RCC system reset(for debug purpose) */ 
  RCC_DeInit(); 
  /* activate HSI (Internal High Speed oscillator) */ 
  RCC_HSICmd(ENABLE); 
  /* Wait until RCC_FLAG_HSIRDY is ready */ 
  while (RCC_GetFlagStatus(RCC_FLAG_HSIRDY) == RESET); 
  /* Enable Prefetch Buffer */ 
  FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable); 
  /* Flash 2 wait state */ FLASH_SetLatency(FLASH_Latency_2); 
  /* HCLK = SYSCLK */ 
  RCC_HCLKConfig(RCC_SYSCLK_Div1);// HCLK = 60 MHz, AHB 
  /* PCLK1 = HCLK/2 */ 
  RCC_PCLK1Config(RCC_HCLK_Div2); // APB1 = 32 MHz 
  /* PCLK2 = HCLK */ 
  RCC_PCLK2Config(RCC_HCLK_Div1); // APB2 = 60 MHz
  /* PLLCLK = (8MHz/2) * 16 = 60 MHz */ 
  RCC_PLLConfig(RCC_PLLSource_HSI_Div2, RCC_PLLMul_15); // 60 MHz
  /* Enable PLL */ RCC_PLLCmd(ENABLE); 
  /* Wait till PLL is ready */ 
  while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET); 
  /* Select PLL as system clock source */ 
  RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK); 
  /* Wait till PLL is used as system clock source */ 
  while(RCC_GetSYSCLKSource() != 0x08);

}