/* can.c */

#include "can.h"

CAN _can;
CAN* can = &_can;

CanTxMsg TxMessage;
CanRxMsg RxMessage;

/* -------------------------------------------------------------------------- */
/* CAN EventHandler
*/
void CanInit(void){
  //set event handlers
  can->Intr0 = 0;
  can->SetIntr0 = CanSetIntr0;
  can->ResetIntr0 = CanResetIntr0;
  can->TxMessage = &TxMessage;
  can->RxMessage = &RxMessage;

  //configurate the alternate-function for can.
  CanConfig();
}
void CanSetIntr0() { can->Intr0 = 1; }
void CanResetIntr0() { can->Intr0 = 0; }

/* -------------------------------------------------------------------------- */
/* CAN function
   - ISO 11898:2003 standard protocol
   - Paramter bit fields
      1) 11 bit type
         SOF | 11bit ID | RTR | ODE | r0 | DLC | 0~8 bytes Data | CRC | ACK | EOF | IFS
      2) 29 bit type
         SOF | 11bit ID | SRR | ODE | 18 bit ID | RTR | r0 | r1 | DLC | 0~8 bytes Data | CRC | ACK | EOF | IFS
      #Package unit description
        SOF: Start of frame
        Identifier: 11-bit identifier to set the priority of the message. The lower the value, the higher the priority.
        RTR: single remote transmission request(Define when you need information from other nodes, Data Frame: 0, Remote Frame:1)
        IDE: dominant single identifier extension(Standard: 0, Extended: 1)
        r0: Reserved bit
        DLC: data length code
        Data: 0~8 bytes
        CRC: Cyclic redundancy check (16bit checksum)
        EOF: End of Frame
        IFS: interframe space (7bit)
        SRR: substitute remote request
        IDE: identifier extension
        r1: Reserved bit
*/

void CanConfig(void)
{
  
  GPIO_InitTypeDef  GPIO_InitStructure;
  
  /* CAN GPIOs configuration **************************************************/
  
  /* 1. Set the interrupt of CAN function. */
  NVIC_InitTypeDef  NVIC_InitStructure;
  NVIC_InitStructure.NVIC_IRQChannel = CAN2_RX0_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Enable GPIO clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);

  /* Set the clock of CAN function. */
  RCC_APB1PeriphClockCmd((RCC_APB1Periph_CAN1 | RCC_APB1Periph_CAN2), ENABLE);
  
  /* Configure CAN RX and TX pins */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  //GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  //GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  
  /* Connect CAN pins to AF9 */
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource12, GPIO_AF_CAN2);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_CAN2); 
  
  /* 3. Initialize the CAN function.*/
  CAN_InitTypeDef        CAN_InitStructure;
  CAN_FilterInitTypeDef  CAN_FilterInitStructure;

  /* CAN register init */
  CAN_DeInit(CAN2);
  CAN_StructInit(&CAN_InitStructure);
  
  /* CAN cell init */
  CAN_InitStructure.CAN_TTCM = DISABLE;
  CAN_InitStructure.CAN_ABOM = DISABLE;
  CAN_InitStructure.CAN_AWUM = DISABLE;
  CAN_InitStructure.CAN_NART = DISABLE;
  CAN_InitStructure.CAN_RFLM = DISABLE;
  CAN_InitStructure.CAN_TXFP = DISABLE; 
  CAN_InitStructure.CAN_Mode=CAN_Mode_Normal;
  
  /* 3-4. Set the baudrate */
  /* CAN Baudrate = 400kbps (CAN clocked at 45 MHz) */
  CAN_InitStructure.CAN_SJW = CAN_SJW_1tq;
  CAN_InitStructure.CAN_BS1 = CAN_BS1_4tq;
  CAN_InitStructure.CAN_BS2 = CAN_BS2_1tq;
  CAN_InitStructure.CAN_Prescaler = 18;
  
  if(CAN_Init(CAN2, &CAN_InitStructure) == CANINITFAILED)
  {
    do {}
    while(1);
  }
  
  /* 3-4. Set the filter */
  /* CAN1 filter number : 0~13*/ 
  /* CAN2 filter number : 14~17   <--- */ 
  CAN_FilterInitStructure.CAN_FilterNumber = 14u;
  CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;
  CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit;
  CAN_FilterInitStructure.CAN_FilterIdHigh = 0x0000;
  CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;
  CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0000;
  CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;  
  CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_FIFO0;
  CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;
  CAN_FilterInit(&CAN_FilterInitStructure);

  CAN_ITConfig(CAN2, CAN_IT_FMP0, ENABLE);
}

void CanSend(uint32_t id,  unsigned char* buf,  unsigned char len){
  
    int i = 0;
    uint8_t TransmitMailbox = 0;
  
    //set default tx message format.
    TxMessage.StdId=id;
    TxMessage.RTR=CAN_RTR_DATA;
    TxMessage.IDE=CAN_ID_STD;
    TxMessage.DLC=len;
    for(i=0;i<len;i++) TxMessage.Data[i] = buf[i];
    CAN_Transmit(CAN2, &TxMessage);
    
    i=0;
    while((CAN_TransmitStatus(CAN2,TransmitMailbox)!= CANTXOK)&& (i!= 0x1F)) i++;
    CAN_CancelTransmit(CAN2,TransmitMailbox);
}

void CAN2_RX0_IRQHandler(void)
{
  CanSetIntr0();
  
  RxMessage.StdId=0x00;
  RxMessage.ExtId=0x00;
  RxMessage.IDE=0;
  RxMessage.DLC=0;
  RxMessage.FMI=0;
  RxMessage.Data[0]=0x00;
  RxMessage.Data[1]=0x00;

  CAN_Receive(CAN2, CAN_FIFO0, &RxMessage);
  
  /* example ===================================================================
  if(can->Intr0){
  {
    can->ResetIntr0();
    if((can->RxMessage->StdId==0x0031) && 
       (can->RxMessage->IDE==CAN_ID_STD)&& 
       (can->RxMessage->DLC==1))
    {
      LedToggle();
    }
  }
  */
}

