/* f429bsp.c */

#include "f429bsp.h"


/* -------------------------------------------------------------------------- */
/* Board Support Package 
*/
BSP _bsp;
BSP* bsp = &_bsp;

void BspInit(void){ 
  bsp->Timer0 = 0;
  bsp->Timer1 = 0;
  bsp->SetTimer0 = SetTimer0;
  bsp->SetTimer1 = SetTimer1;
  bsp->ResetTimer0 = ResetTimer0;
  bsp->ResetTimer1 = ResetTimer1;
}
void SetTimer0() { bsp->Timer0 = 1; }
void SetTimer1() { bsp->Timer1 = 1; }
void ResetTimer0() { bsp->Timer0 = 0; }
void ResetTimer1() { bsp->Timer1 = 0; }

/* -------------------------------------------------------------------------- */
/* Timer 1~3 
    - Timer 3, Use 'Bsp::Timer0'
*/ 
void TIM3_IRQHandler(void)
{
  if (TIM_GetITStatus(TIM3, TIM_IT_Update) != RESET)
  {
    TIM_ClearITPendingBit(TIM3, TIM_IT_Update);

    SetTimer0();
  }
}


void TimerInit(void)
{
  /* Enable the TIM3 gloabal Interrupt ---------------------------------------*/
  NVIC_InitTypeDef          NVIC_InitStructure;
  NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;// this sets the priority group of the USART1 interrupts
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2; // this sets the subpriority inside the group
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;  // the USART1 interrupts are globally enabled
  NVIC_Init(&NVIC_InitStructure);    // the properties are passed to the NVIC_Init function which takes care of the low level stuff
  NVIC_SetPriority(TIM3_IRQn,2);

 
  /* Setting TIM3 Timer-------------------------------------------------------*/
  /*
     -> Current APB1Periph Clock 72MHz / Target Tick 50ms
        = clock / ((period + 1) * (prescaler + 1)) 
        = 72000000 / 3600*1000
        = 72000000 / 3600000 = 20Hz = 1/20s = 0.05s
        = 50ms
  */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
 
  TIM_TimeBaseInitTypeDef   TIM_TimeBaseStructure;
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
#if 0 /* PCLK1 Timer - Timer Clock 72MHz -> 50ms Tick */
  TIM_TimeBaseStructure.TIM_Period = 3600 - 1;      //50msec 
  TIM_TimeBaseStructure.TIM_Prescaler = 1000 - 1;
#endif
  /* PCLK1 Timer - Timer Clock 90MHz -> 45ms Tick */
  TIM_TimeBaseStructure.TIM_Period = 4500 - 1;      //50msec 
  TIM_TimeBaseStructure.TIM_Prescaler = 1000 - 1;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

  TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE);
  TIM_Cmd(TIM3, ENABLE);
}


/* -------------------------------------------------------------------------- */
/* UserClockSetting
    - Custom Clock, for CAN 
      1) SysClock  => 144MHz
      2) HCLK      => 72MHz
      3) APB1Periph PCLK1 Peripheral Clock => 36MHz
      4) APB1Periph PCLK1 Timer Clock      => 72MHz
      5) APB2Periph PCLK2 Peripheral Clock => 36MHz
      6) APB2Periph PCLK2 Timer Clock      => 72MHz
*/
void UserClockSetting()
{
  ErrorStatus HSEStartUpStatus;
  
  RCC_DeInit();
  RCC_HSEConfig(RCC_HSE_ON);
  HSEStartUpStatus = RCC_WaitForHSEStartUp(); 
  
  if(HSEStartUpStatus == SUCCESS)
  {
    FLASH_PrefetchBufferCmd(ENABLE);
    FLASH_SetLatency(FLASH_Latency_3);  
    RCC_PLLConfig(RCC_PLLSource_HSI,8,144,2,4); 
    RCC_HCLKConfig(RCC_SYSCLK_Div2); 
    RCC_PCLK2Config(RCC_HCLK_Div2); 
    RCC_PCLK1Config(RCC_HCLK_Div2);
    RCC_PLLCmd(ENABLE); 
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY)==RESET)  { }  

    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
    while(RCC_GetSYSCLKSource() !=0x08) { }   
  }
}
